# Analyzing the overall effect of context (ignoring nonlinearities) 
# in the three context experiments

setwd(dirname(rstudioapi::getActiveDocumentContext()$path))  

library(ggplot2)
library(brms)

df = read.csv('data/pooled.csv')

## OBSERVED
a = aggregate(response ~ file + measure, df, mean)
ggplot(a, aes(x = file, y = response, color = measure, shape = measure)) +
  geom_point() +
  coord_flip() +
  theme_bw()


a1 = aggregate(response ~ emGroup + measure, df, function(x) mean(x) * 100)
a1$valIntens = ifelse(a1$measure %in% c('intens', 'intensNeg', 'intensPos'), 'Intensity', 'Valence')
a1$Context = ifelse(a1$measure %in% c('intensNeg', 'valNeg'), 'Negative', 'Positive')
a1$Context = ifelse(a1$measure %in% c('val', 'intens'), 'None', a1$Context)
a1$valIntens = factor(a1$valIntens, levels = c('Valence', 'Intensity'))
a1$em = as.character(a1$emGroup)
a1$em[a1$emGroup == 'surprise'] = 'Gasps\n(fright / surprise)'
a1$em[a1$emGroup == 'pls-pain'] = 'Moans\n(pain / pleasure)'
a1$em[a1$emGroup == 'del-fear'] = 'Screams\n(fear / delight)'
a1$em[a1$emGroup == 'ach-frust'] = 'Roars\n(frustration / achievement)'
a1$em = as.factor(a1$em)

ggplot(a1, aes(x = em, y = response, color = Context, shape = Context)) +
  geom_point(size = 3) +
  scale_color_manual(values = c('gray25', 'gray50', 'gray75')) +
  coord_flip() +
  facet_wrap(~valIntens) +
  xlab('') + ylab('Rating, %') +
  theme_bw()
ggsave('pix/context_obs.png', width = 20, height = 8, units = 'cm', dpi = 300)


## MODEL
df = droplevels(df[!is.na(df$response), ])  # in Exp 3 we get NA when participants skip a sound

mod_nonlin = readRDS('mod/mod_nonlin.RDS')

newdata = expand.grid(
  emGroup = levels(df$emGroup),
  cond = levels(df$cond),
  measure = levels(df$measure)
)
fit = fitted(mod_nonlin, newdata = newdata, re_formula = NA, robust = TRUE, summary = FALSE)

# get fitted values averaging over all 3 levels of nonlinearities ("cond")
fit1 = expand.grid(
  emGroup = levels(df$emGroup),
  measure = levels(df$measure),
  fit = NA, lwr = NA, upr = NA
)
for (i in 1:nrow(fit1)) {
  idx = which(newdata$emGroup == fit1$emGroup[i] &
                newdata$measure == fit1$measure[i])
  av = rowMeans(fit[, idx]) * 100  # to %
  fit1[i, 3:5] = quantile(av, probs = c(.5, .025, .975))
}

fit1_av = expand.grid(
  emGroup = 'Overall',
  measure = levels(df$measure),
  fit = NA, lwr = NA, upr = NA
)
for (i in 1:nrow(fit1_av)) {
  idx = which(newdata$measure == fit1_av$measure[i])
  av = rowMeans(fit[, idx]) * 100  # to %
  fit1_av[i, 3:5] = quantile(av, probs = c(.5, .025, .975))
}

fit1 = rbind(fit1, fit1_av)
fit1$valIntens = ifelse(fit1$measure %in% c('intens', 'intensNeg', 'intensPos'), 'Intensity', 'Valence')
fit1$Context = ifelse(fit1$measure %in% c('intensNeg', 'valNeg'), 'Negative', 'Positive')
fit1$Context = ifelse(fit1$measure %in% c('intens', 'val'), 'None', fit1$Context)
fit1$valIntens = factor(fit1$valIntens, levels = c('Valence', 'Intensity'))
fit1$em = as.character(fit1$emGroup)
fit1$em[fit1$emGroup == 'surprise'] = 'Gasps\n(fright / surprise)'
fit1$em[fit1$emGroup == 'pls-pain'] = 'Moans\n(pain / pleasure)'
fit1$em[fit1$emGroup == 'del-fear'] = 'Screams\n(fear / delight)'
fit1$em[fit1$emGroup == 'ach-frust'] = 'Roars\n(frustration / achievement)'
fit1$em = factor(fit1$em, levels = rev(c('Overall', 'Gasps\n(fright / surprise)', 'Moans\n(pain / pleasure)', 'Roars\n(frustration / achievement)', 'Screams\n(fear / delight)')))
fit1$ci = soundgen:::reportCI(fit1[, c('fit', 'lwr', 'upr')], digits = 1)

ggplot(fit1, aes(x = em, y = fit, ymin = lwr, ymax = upr, color = Context, shape = Context)) +
  geom_errorbar(width = 0, position = position_dodge(.3)) +
  geom_point(size = 3, position = position_dodge(.3)) +
  geom_vline(xintercept = 4.5, linetype = 3) +
  scale_color_manual(values = c('gray25', 'gray50', 'gray75')) +
  coord_flip() +
  facet_wrap(~valIntens) +
  xlab('') + ylab('Rating, %') +
  theme_bw() +
  theme(legend.position = 'right',
        panel.grid = element_blank())
ggsave('pix/context_mod.png', width = 20, height = 8, units = 'cm', dpi = 600)

fit1[, c('emGroup', 'measure', 'ci')]

# contrasts (pos_vs_neg)
newdata$valIntens = ifelse(newdata$measure %in% c('intensNeg', 'intensPos'), 'Intensity', 'Valence')
newdata$Context = ifelse(newdata$measure %in% c('intensNeg', 'valNeg'), 'Negative', 'Positive')
newdata$Context = ifelse(newdata$measure == 'val', 'None', newdata$Context)
fit2 = expand.grid(
  emGroup = levels(df$emGroup),
  valIntens = c('Valence', 'Intensity'),
  pos_vs_neg = NA
)
for (i in 1:nrow(fit2)) {
  idx = which(newdata$emGroup == fit2$emGroup[i] &
                newdata$valIntens == fit2$valIntens[i])
  idx_pos = idx[which(newdata$Context[idx] == 'Positive')]
  idx_neg = idx[which(newdata$Context[idx] == 'Negative')]
  contrast = (fit[, idx_pos] - fit[, idx_neg]) * 100  # to %
  contrast = rowMeans(contrast)
  fit2$pos_vs_neg[i] = soundgen:::reportCI(quantile(contrast, probs = c(.5, .025, .975)), digits = 1)
}
fit2


## How much does the context affect the valence of individual sounds (Exp 2), 
# and which ones "flip" due to context?
newdata = expand.grid(
  measure = c('valPos', 'valNeg'),
  file = levels(df$file)
)
newdata[, c('emGroup', 'cond')] = df[match(newdata$file, df$file), c('emGroup', 'cond')]
fit = fitted(mod_nonlin, newdata = newdata, re_formula = '~(1|file)', summary = FALSE) * 100  # to %

contrasts = data.frame(
  file = levels(df$file),
  fit = NA, lwr = NA, upr = NA,
  fit_pos = NA, fit_neg = NA
)
for (i in 1:nrow(contrasts)) {
  idx = which(newdata$file == contrasts$file[i])
  idx_pos = idx[which(newdata$measure[idx] == 'valPos')]
  idx_neg = idx[which(newdata$measure[idx] == 'valNeg')]
  pos_vs_neg = fit[, idx_pos] - fit[, idx_neg] 
  contrasts[i, c('fit', 'lwr', 'upr')] = quantile(pos_vs_neg, probs = c(.5, .025, .975))
  contrasts$fit_pos[i] = median(fit[, idx_pos])
  contrasts$fit_neg[i] = median(fit[, idx_neg])
}

contrasts$flip = contrasts$fit_pos > 50 & contrasts$fit_neg < 50
table(contrasts$flip)  # 144/144 - all stimuli flip

# obs
a = aggregate(response ~ file + measure, droplevels(df[df$exp == 'exp2', ]), mean)
a1 = reshape2::dcast(a, file ~ measure)
a1$flip = a1$valPos > .5 & a1$valNeg < .5
table(a1$flip)  # 122 out of 144 flip
a1[!a1$flip, ]
table(a1$valPos[!a1$flip] > .5)  # 21/22 non-flipped are always negative, 1 always positive

# + add plot if needed