# Analyzing the effect of nonlinearities in the three context experiments

setwd(dirname(rstudioapi::getActiveDocumentContext()$path))  

library(ggplot2)
library(brms)

df = read.csv('data/pooled.csv')

df = droplevels(df[!is.na(df$response), ])  # in Exp 3 we get NA when participants skip a sound

## Model selection
if (FALSE) {
  df$resp01 = (df$response + .01) / 1.02
  range(df$resp01)
  # mod1 = brm(resp01 ~ emGroup * cond * measure + (1|id) + (1|file) + (cond|sound), data = df, family = 'beta', cores = 4, chains = 4, warmup = 250, iter = 1000)  # ~2 h / 2000 iter; rerun!
  # saveRDS(mod1, 'mod/temp_mod1.RDS')
  mod1 = readRDS('mod/temp_mod1.RDS')
  
  summary(mod1)
  pp_check(mod1)
  
  ## zero-one-inflated beta (ZOIB): see https://vuorre.netlify.com/post/2019/02/18/analyze-analog-scale-ratings-with-zero-one-inflated-beta-models/
  # zoib_model <- bf(
  #   Rating ~ group,
  #   phi ~ group,
  #   zoi ~ group,
  #   coi ~ group, 
  #   family = zero_one_inflated_beta()
  # )
  # 
  # The four sub-models of our model are, in order of appearance: 1. the model of the beta distribution’s mean (read, “predict Rating’s mean from group”). Then, 2. the model of phi; the beta distribution’s precision. 3. zoi is the zero-one inflation (that is, we model the probability of a binary rating as a function of group). 4. coi is the conditional one-inflation: Given that a response was {0, 1}, the probability of it being 1 is modelled on group.
  
  mod2 = brm(bf(response ~ emGroup * cond * measure + (1|id) + (1|file) + (cond|sound),
                zoi ~ emGroup * cond * measure,
                coi ~ emGroup * cond * measure),
             data = df, family = 'zero_one_inflated_beta',
             cores = 4, chains = 4, warmup = 250, iter = 1000)
  saveRDS(mod2, 'mod/temp_mod2.RDS')
  # summary(mod2)  # coi didn't converge, >20 h / 1000 iterations (!)
  pp_check(mod2)
  plot(mod2)
  stancode(mod2)
  # could try to set a strong conservative prior on b_coi:
  prior_summary(mod2)
  brms::make_stancode(bf(response ~ emGroup * cond * measure + (1|id) + (1|file) + (cond|sound),
                         zoi ~ emGroup * cond * measure,
                         coi ~ emGroup * cond * measure),
                      data = df, family = 'zero_one_inflated_beta', prior = c(set_prior('logistic(0, 1)', class = 'b', dpar = 'coi'), set_prior('logistic(0, 1)', class = 'b', dpar = 'zoi')))
  
  mod3 = brm(bf(response ~ emGroup * cond * measure + (1|id) + (1|file) + (cond|sound),
                zoi ~ emGroup * cond * measure),
             data = df, family = 'zero_one_inflated_beta',
             cores = 4, chains = 4, warmup = 250, iter = 1000)  # need ~6000 iter
  saveRDS(mod3, 'mod/temp_mod3.RDS')
  pp_check(mod3)
  summary(mod3)
  
  # loo(mod1, mod2)
  # waic(mod2, mod3)  # actually, mod2 is still much better, despite convergence issues
  
  mod4 = brm(bf(response ~ emGroup * cond * measure + (1|id) + (1|file) + (cond|sound),
                zoi ~ emGroup * cond * measure,
                coi ~ emGroup * cond * measure),
             data = df, family = 'zero_one_inflated_beta',
             cores = 4, chains = 4, warmup = 100, iter = 300,
             prior = c(set_prior('logistic(0, 1)', class = 'b', dpar = 'coi'), 
                       set_prior('logistic(0, 1)', class = 'b', dpar = 'zoi')))
  saveRDS(mod4, 'mod/temp_mod4.RDS')  # 6 h / 300 iter, so no faster than mod2
  mod4 = readRDS('mod/temp_mod4.RDS')
  # summary(mod4)  # 
  plot(mod4, pars = '?coi')  # much better convergence with these priors!
  pp_check(mod4)
  # waic(mod2, mod4)  # of course, same as mod2
  
  df = droplevels(df[sample(1:nrow(df), size = 5000, replace = FALSE), ])
  
  mod_nonlin1 = brm(bf(
    response ~ emGroup * cond * measure + (1|id) + (1|file) + (cond|sound),
    zoi ~ emGroup * cond * measure,
    coi ~ emGroup * cond * measure),
    data = df, family = 'zero_one_inflated_beta',
    cores = 4, chains = 4, warmup = 500, iter = 2000,
    prior = c(set_prior('logistic(0, 1)', class = 'b', dpar = 'coi'),
              set_prior('logistic(0, 1)', class = 'b', dpar = 'zoi')))
  
  mod_nonlin2 = brm(bf(
    response ~ emGroup * cond * measure + (1|id) + (1|file) + (cond * measure|sound),
    zoi ~ emGroup * cond * measure,
    coi ~ emGroup * cond * measure),
    data = df, family = 'zero_one_inflated_beta',
    cores = 4, chains = 4, warmup = 500, iter = 2000,
    prior = c(set_prior('logistic(0, 1)', class = 'b', dpar = 'coi'),
              set_prior('logistic(0, 1)', class = 'b', dpar = 'zoi')))
  
  waic(mod_nonlin1, mod_nonlin2)  # mod_nonlin2 much better
}

## Final model - check mod2/mod3 in terms of predictions (fitted values)
mod_nonlin = brm(bf(
  response ~ emGroup * cond * measure + (1|id) + (1|file) + (cond * measure|sound),
  zoi ~ emGroup * cond * measure,
  coi ~ emGroup * cond * measure),
  data = df, family = 'zero_one_inflated_beta',
  cores = 4, chains = 4, warmup = 500, iter = 2000,
  prior = c(set_prior('logistic(0, 1)', class = 'b', dpar = 'coi'),
            set_prior('logistic(0, 1)', class = 'b', dpar = 'zoi')))
# saveRDS(mod_nonlin, 'mod/mod_nonlin.RDS')  # ~15 h / 2000 iter
mod_nonlin = readRDS('mod/mod_nonlin.RDS')
# summary(mod_nonlin)
# pp_check(mod_nonlin)

newdata = expand.grid(
  emGroup = levels(df$emGroup),
  cond = levels(df$cond),
  measure = levels(df$measure)
)
fit = fitted(mod_nonlin, newdata = newdata, re_formula = NA, robust = TRUE) * 100  # to %
colnames(fit) = c('fit', 'se', 'lwr', 'upr')
df_plot = cbind(newdata, fit)
df_plot$measure = factor(df_plot$measure, levels = c('val', 'valNeg', 'valPos', 'intens', 'intensNeg', 'intensPos'))

ggplot(df_plot, aes(x = emGroup, y = fit, ymin = lwr, ymax = upr, color = cond, shape = cond)) +
  geom_point(size = 2, position = position_dodge(.4)) +
  geom_errorbar(width = 0, position = position_dodge(.4)) +
  coord_flip() +
  facet_wrap(~measure) +
  theme_bw()
# ggsave('pix/nonlin_mod.png', width = 25, height = 15, units = 'cm', dpi = 300)

# contrasts
fit1 = fitted(mod_nonlin, newdata = newdata, re_formula = NA, summary = FALSE) * 100  # to %

contrasts1 = contrasts2 = expand.grid(
  emGroup = levels(df$emGroup),
  measure = levels(df$measure),
  fit = NA, lwr = NA, upr = NA
)
for (i in 1:nrow(contrasts1)) {
  idx = which(newdata$emGroup == contrasts1$emGroup[i] &
                newdata$measure == contrasts1$measure[i])
  idx_none = idx[which(newdata$cond[idx] == 'none')]
  idx_subh = idx[which(newdata$cond[idx] == 'subh')]
  idx_chaos = idx[which(newdata$cond[idx] == 'chaos')]
  subh_vs_none = fit1[, idx_subh] - fit1[, idx_none]
  contrasts1[i, 3:5] = quantile(subh_vs_none, probs = c(.5, .025, .975))
  chaos_vs_none = fit1[, idx_chaos] - fit1[, idx_none]
  contrasts2[i, 3:5] = quantile(chaos_vs_none, probs = c(.5, .025, .975))
}
contrasts1$Manipulation = 'Subharmonics / sidebands'
contrasts2$Manipulation = 'Chaos'

contrasts_av1 = contrasts_av2 = data.frame(
  measure = levels(df$measure),
  fit = NA, lwr = NA, upr = NA
)
for (i in 1:nrow(contrasts_av1)) {
  idx = which(newdata$measure == contrasts_av1$measure[i])
  idx_none = idx[which(newdata$cond[idx] == 'none')]
  idx_subh = idx[which(newdata$cond[idx] == 'subh')]
  idx_chaos = idx[which(newdata$cond[idx] == 'chaos')]
  subh_vs_none = rowMeans(fit1[, idx_subh] - fit1[, idx_none])
  contrasts_av1[i, 2:4] = quantile(subh_vs_none, probs = c(.5, .025, .975))
  chaos_vs_none = rowMeans(fit1[, idx_chaos] - fit1[, idx_none])
  contrasts_av2[i, 2:4] = quantile(chaos_vs_none, probs = c(.5, .025, .975))
}
contrasts_av1$Manipulation = 'Subharmonics / sidebands'
contrasts_av2$Manipulation = 'Chaos'
contrasts_av = rbind(contrasts_av1, contrasts_av2)
contrasts_av$emGroup = 'Overall'
contrasts_av = contrasts_av[, c('emGroup', 'measure', 'fit', 'lwr', 'upr', 'Manipulation')]

contrasts = rbind(contrasts1, contrasts2, contrasts_av)
contrasts$Manipulation = factor(contrasts$Manipulation, levels = c('Subharmonics / sidebands', 'Chaos'))
for (i in 1:nrow(contrasts)) {
  contrasts$Measure[i] = switch(
    as.character(contrasts$measure[i]),
    'val' = 'Valence: no context',
    'valNeg' = 'Valence: neg. context',
    'valPos' = 'Valence: pos. context',
    'intens' = 'Intensity: no context',
    'intensNeg' = 'Intensity: neg. context',
    'intensPos' = 'Intensity: pos. context'
  )
  contrasts$emBlock[i] = switch(
    as.character(contrasts$emGroup[i]),
    'Overall' = 'Overall',
    'surprise' = 'Gasps\n(fright / surprise)',
    'pls-pain' = 'Moans\n(pain / pleasure)',
    'del-fear' = 'Screams\n(fear / delight)',
    'ach-frust' = 'Roars\n(frustration / achievement)'
  )
}
contrasts$Measure = factor(contrasts$Measure, levels = c('Valence: no context', 'Valence: neg. context', 'Valence: pos. context', 'Intensity: no context', 'Intensity: neg. context', 'Intensity: pos. context'))
contrasts$emBlock = factor(contrasts$emBlock, levels = rev(c('Overall', 'Gasps\n(fright / surprise)', 'Moans\n(pain / pleasure)', 'Roars\n(frustration / achievement)', 'Screams\n(fear / delight)')))
contrasts$ci = soundgen:::reportCI(contrasts[, c('fit', 'lwr', 'upr')], digits = 1)

ggplot(contrasts, aes(x = emBlock, y = fit, ymin = lwr, ymax = upr, color = Manipulation, shape = Manipulation)) +
  geom_point(size = 2, position = position_dodge(.4)) +
  geom_errorbar(width = 0, position = position_dodge(.4)) +
  geom_hline(yintercept = 0, linetype = 3) +
  geom_vline(xintercept = 4.5, linetype = 3) +
  xlab('') + ylab('Effect vs. no nonlinearities, %') +
  coord_flip() +
  facet_wrap(~Measure) +
  theme_bw() +
  theme(legend.position = 'top', 
        panel.grid = element_blank())
# ggsave('pix/nonlin_effect_per_measure_and_emGroup_mod.png', width = 20, height = 12, units = 'cm', dpi = 600)

contrasts[, c('emGroup', 'measure', 'Manipulation', 'ci')]



## Double contrasts - neg/no, pos/no, neg/pos contexts
c1 = expand.grid(
  emGroup = levels(df$emGroup),
  nlp = c('subh', 'chaos'),
  type = c('val', 'intens'),
  contrast = c('neg_none', 'pos_none', 'neg_pos'),
  fit = NA, lwr = NA, upr = NA
)
c1$measure1 = rep(c('valNeg', 'intensNeg', 'valPos', 'intensPos', 'valNeg', 'intensNeg'), each = 8)
c1$measure2 = rep(c('val', 'intens', 'val', 'intens', 'valPos', 'intensPos'), each = 8)

for (i in 1:nrow(c1)) {
  idx1 = which(newdata$emGroup == c1$emGroup[i] &
                 as.character(newdata$cond) == as.character(c1$nlp[i]) & 
                 newdata$measure == c1$measure1[i])
  idx1_none = which(newdata$emGroup == c1$emGroup[i] &
                      as.character(newdata$cond) == 'none' & 
                      newdata$measure == c1$measure1[i])
  idx2 = which(newdata$emGroup == c1$emGroup[i] &
                 as.character(newdata$cond) == as.character(c1$nlp[i]) & 
                 newdata$measure == c1$measure2[i])
  idx2_none = which(newdata$emGroup == c1$emGroup[i] &
                      as.character(newdata$cond) == 'none' & 
                      newdata$measure == c1$measure2[i])
  con1 = fit1[, idx1] - fit1[, idx1_none]
  con2 = fit1[, idx2] - fit1[, idx2_none]
  c1[i, c('fit', 'lwr', 'upr')] = quantile(con1 - con2, probs = c(.5, .025, .975))
}

c1_av = expand.grid(
  nlp = c('subh', 'chaos'),
  type = c('val', 'intens'),
  contrast = c('neg_none', 'pos_none', 'neg_pos'),
  fit = NA, lwr = NA, upr = NA
)
c1_av$measure1 = rep(c('valNeg', 'intensNeg', 'valPos', 'intensPos', 'valNeg', 'intensNeg'), each = 2)
c1_av$measure2 = rep(c('val', 'intens', 'val', 'intens', 'valPos', 'intensPos'), each = 2)

for (i in 1:nrow(c1_av)) {
  idx1 = which(as.character(newdata$cond) == as.character(c1_av$nlp[i]) & 
                 newdata$measure == c1_av$measure1[i])
  idx1_none = which(as.character(newdata$cond) == 'none' & 
                      newdata$measure == c1_av$measure1[i])
  idx2 = which(as.character(newdata$cond) == as.character(c1_av$nlp[i]) & 
                 newdata$measure == c1_av$measure2[i])
  idx2_none = which(as.character(newdata$cond) == 'none' & 
                      newdata$measure == c1_av$measure2[i])
  con1 = rowMeans(fit1[, idx1] - fit1[, idx1_none])
  con2 = rowMeans(fit1[, idx2] - fit1[, idx2_none])
  c1_av[i, c('fit', 'lwr', 'upr')] = quantile(con1 - con2, probs = c(.5, .025, .975))
}

c1_av$emGroup = 'Overall'
c1_av = c1_av[, c('emGroup', 'nlp', 'type', 'contrast', 'fit', 'lwr', 'upr', 'measure1', 'measure2')]

c2 = rbind(c1, c1_av)

c2$Manipulation = ifelse(c2$nlp == 'subh', 'Subharmonics / sidebands', 'Chaos')
c2$Manipulation = factor(c2$Manipulation, levels = c('Subharmonics / sidebands', 'Chaos'))
c2$temp = paste0(c2$type, ':', c2$contrast)
# unique(c1$temp)
for (i in 1:nrow(c2)) {
  c2$Measure[i] = switch(
    as.character(c2$temp[i]),
    'val:neg_none' = 'Valence: neg. vs none',
    'val:pos_none' = 'Valence: pos. vs none',
    'val:neg_pos' = 'Valence: neg. vs pos.',
    'intens:neg_none' = 'Intensity: neg. vs none',
    'intens:pos_none' = 'Intensity: pos. vs none',
    'intens:neg_pos' = 'Intensity: neg. vs pos.'
  )
  c2$emBlock[i] = switch(
    as.character(c2$emGroup[i]),
    'Overall' = 'Overall',
    'surprise' = 'Gasps\n(fright / surprise)',
    'pls-pain' = 'Moans\n(pain / pleasure)',
    'del-fear' = 'Screams\n(fear / delight)',
    'ach-frust' = 'Roars\n(frustration / achievement)'
  )
}
c2$Measure = factor(c2$Measure, levels = c('Valence: neg. vs none', 'Valence: pos. vs none', 'Valence: neg. vs pos.', 'Intensity: neg. vs none', 'Intensity: pos. vs none', 'Intensity: neg. vs pos.'))
c2$emBlock = factor(c2$emBlock, levels = rev(c('Overall', 'Gasps\n(fright / surprise)', 'Moans\n(pain / pleasure)', 'Roars\n(frustration / achievement)', 'Screams\n(fear / delight)')))
c2$ci = soundgen:::reportCI(c2[, c('fit', 'lwr', 'upr')], digits = 1)

ggplot(c2, aes(x = emBlock, y = fit, ymin = lwr, ymax = upr, color = Manipulation, shape = Manipulation)) +
  geom_point(size = 2, position = position_dodge(.4)) +
  geom_errorbar(width = 0, position = position_dodge(.4)) +
  geom_hline(yintercept = 0, linetype = 3) +
  geom_vline(xintercept = 4.5, linetype = 3) +
  xlab('') + ylab('Difference in the effect of NLPs, %') +
  coord_flip() +
  facet_wrap(~Measure) +
  theme_bw() +
  theme(legend.position = 'top', 
        panel.grid = element_blank())
# ggsave('pix/nonlin_double_contrasts.png', width = 20, height = 12, units = 'cm', dpi = 600)

c2[, c('emGroup', 'Manipulation', 'Measure', 'ci')]


## Visualize the effect of nonlin for individual sounds
newdata = expand.grid(
  measure = levels(df$measure),
  file = levels(df$file)
)
newdata[, c('emGroup', 'cond', 'sound')] = df[match(newdata$file, df$file), c('emGroup', 'cond', 'sound')]
fit = fitted(mod_nonlin, newdata = newdata, re_formula = '~(1|file) + (cond|sound)', robust = TRUE) * 100  # to %
colnames(fit) = c('fit', 'se', 'lwr', 'upr')
di = cbind(newdata, fit)
head(di)  # 15 rows for each of 48 sounds: 5 measures * 3 manipulations

di1 = reshape2::dcast(di, sound + emGroup + measure ~ cond, value.var = 'fit')
head(di1)  # 5 rows per sound: 5 measures

di1$subh_vs_none = di1$sub - di1$none
di1$chaos_vs_none = di1$chaos - di1$none
boxplot(subh_vs_none ~ measure, di1)
boxplot(chaos_vs_none ~ measure, di1)

ggplot(di1, aes(x = emGroup, y = subh_vs_none)) +
  geom_violin() +
  geom_hline(yintercept = 0, linetype = 3) +
  facet_wrap(~measure) +
  coord_flip() +
  theme_bw()

ggplot(di1, aes(x = emGroup, y = chaos_vs_none)) +
  geom_violin() +
  geom_hline(yintercept = 0, linetype = 3) +
  facet_wrap(~measure) +
  coord_flip() +
  theme_bw()

# cf to observed
a = aggregate(response ~ measure + sound + cond, df, mean)
a1 = reshape2::dcast(a, sound + measure ~ cond, value.var = 'response')
a1$subh_vs_none = a1$sub - a1$none
a1$chaos_vs_none = a1$chaos - a1$none
boxplot(subh_vs_none ~ measure, a1)
boxplot(chaos_vs_none ~ measure, a1)




## How ambiguous are the individual sounds w/o context (Exp 1), 
# and which ones "flip" due to nonlin in Exp 1?
newdata = expand.grid(
  measure = 'val',
  file = levels(df$file)
)
newdata[, c('emGroup', 'cond', 'sound')] = df[match(newdata$file, df$file), c('emGroup', 'cond', 'sound')]
fit = fitted(mod_nonlin, newdata = newdata, re_formula = '~(1|file) + (cond|sound)', robust = TRUE) * 100  # to %
colnames(fit) = c('fit', 'se', 'lwr', 'upr')
di = cbind(newdata, fit)
head(di)

di1 = reshape2::dcast(di, sound + emGroup ~ cond, value.var = 'fit')
di1$flip = apply(di1[, c('chaos', 'none', 'subh')], 1, function(x) max(x) > 50 & min(x) < 50)
table(di1$flip)  # 17/48
table(di1$flip, di1$emGroup)  
# 4/7 roars, 5/8 screams, 6/26 moans, and 2/7 gasps "flip" their valence 
# around 50% depending on which nonlinear phenomena are present

di$flip = di1$flip[match(di$sound, di1$sound)]
di$cl = ifelse(di$flip, 'black', 'gray80')
di$Manipulation = as.character(di$cond)
di$Manipulation[which(di$cond == 'none')] = 'None'
di$Manipulation[di$cond == 'subh'] = 'Subh / sidebands'
di$Manipulation[di$cond == 'chaos'] = 'Chaos'
di$Manipulation = factor(di$Manipulation, levels = c('None', 'Subh / sidebands', 'Chaos'))
di$sound = factor(di$sound, levels = rev(levels(di$sound)))

ggplot(di, aes(x = sound, y = fit, ymin = lwr, ymax = upr, shape = Manipulation, col = Manipulation, alpha = cl)) +
  geom_errorbar(width = 0, position = position_dodge(.5)) +
  geom_point(size = 2, position = position_dodge(.5)) + 
  geom_hline(yintercept = 50, linetype = 2) +
  geom_vline(xintercept = 7.5, linetype = 3) +
  geom_vline(xintercept = 33.5, linetype = 3) +
  geom_vline(xintercept = 41.5, linetype = 3) +
  annotate('text', x = 45, y = 65, label = 'Roars') +
  annotate('text', x = 21, y = 65, label = 'Moans') +
  annotate('text', x = 37.5, y = 65, label = 'Screams') +
  annotate('text', x = 3.5, y = 65, label = 'Gasps') +
  coord_flip() +
  scale_alpha_manual(values = c(1, .2), guide = FALSE) +
  scale_y_continuous(name = 'Valence without context, %', breaks = seq(0, 100, by = 10)) +
  xlab('') +
  theme_bw() +
  theme(panel.grid = element_blank(),
        legend.position = 'top')

ggsave('pix/exp1_indiv_sounds.png', width = 15, height = 20, units = 'cm', dpi = 300)

# Cf. to observed (valence, exp 1)
d1 = droplevels(df[df$exp == 'exp1' & df$slider == 'valence', ])

a = aggregate(response ~ sound + cond, d1, mean)  # or median
ggplot(a, aes(x = sound, y = response, shape = cond, col = cond)) +
  geom_point() + 
  coord_flip() +
  theme_bw()

# which sound flip around 50% depending on nonlin?
a1 = reshape2::dcast(a, sound ~ cond)
a1$flip = apply(a1[, 2:4], 1, function(x) max(x) > .5 & min(x) < .5)
a1$emGroup = d1$emGroup[match(a1$sound, d1$sound)]
table(a1$flip)  # 16/48
table(a1$flip, a1$emGroup)
a1[a1$flip, ]  # 16 out of 48 "flip", incl. 1/7 roars, 2/8 screams, 8/26 moans, and 5/7 gasps

d2 = droplevels(df[df$exp == 'exp2', ])
a2_pos = aggregate(response ~ sound + cond, d2[d2$context == 'pos', ], mean)
a2_neg = aggregate(response ~ sound + cond, d2[d2$context == 'neg', ], mean)

a22_pos = reshape2::dcast(a2_pos, sound ~ cond)
a22_pos$flip = apply(a22_pos[, 2:4], 1, function(x) max(x) > .5 & min(x) < .5)
a22_pos$emGroup = d1$emGroup[match(a22_pos$sound, d2$sound)]
table(a22_pos$flip)  # 9/48 flip in a positive context in Exp 2
table(a22_pos$flip, a22_pos$emGroup)

a22_neg = reshape2::dcast(a2_neg, sound ~ cond)
a22_neg$flip = apply(a22_neg[, 2:4], 1, function(x) max(x) > .5 & min(x) < .5)
a22_neg$emGroup = d1$emGroup[match(a22_neg$sound, d2$sound)]
table(a22_neg$flip)  # 1/48 flips in a negative context in Exp 2
table(a22_neg$flip, a22_neg$emGroup)


## Contrasts per sound in Experiment 3 (intensity)
newdata = expand.grid(
  measure = c('intensPos', 'intensNeg'),
  file = levels(df$file)
)
newdata[, c('emGroup', 'cond', 'sound')] = df[match(newdata$file, df$file), c('emGroup', 'cond', 'sound')]
fit = fitted(mod_nonlin, newdata = newdata, re_formula = '~(1|file) + (cond|sound)', summary = FALSE) * 100  # to %

contrasts1 = contrasts2 = expand.grid(
  sound = levels(df$sound),
  measure = c('intensPos', 'intensNeg'),
  fit = NA, lwr = NA, upr = NA
)
for (i in 1:nrow(contrasts1)) {
  idx = which(newdata$sound == contrasts1$sound[i] &
                newdata$measure == contrasts1$measure[i])
  idx_none = idx[which(newdata$cond[idx] == 'none')]
  idx_subh = idx[which(newdata$cond[idx] == 'subh')]
  idx_chaos = idx[which(newdata$cond[idx] == 'chaos')]
  subh_vs_none = fit[, idx_subh] - fit[, idx_none]
  contrasts1[i, c('fit', 'lwr', 'upr')] = quantile(subh_vs_none, probs = c(.5, .025, .975))
  chaos_vs_none = fit[, idx_chaos] - fit[, idx_none]
  contrasts2[i, c('fit', 'lwr', 'upr')] = quantile(chaos_vs_none, probs = c(.5, .025, .975))
}

contrasts1$Manipulation = 'Subharmonics'
contrasts2$Manipulation = 'Chaos'
contrasts = rbind(contrasts1, contrasts2)
contrasts$ci = soundgen:::reportCI(contrasts[, c('fit', 'lwr', 'upr')], digits = 1)

a = aggregate(response ~ sound + measure + cond, droplevels(df[df$exp == 'exp3', ]), mean)
a1 = reshape2::dcast(a, sound + measure ~ cond)
a1$Subharmonics = (a1$subh - a1$none) * 100
a1$Chaos = (a1$chaos - a1$none) * 100
a2 = reshape2::melt(a1[, c('sound', 'measure', 'Subharmonics', 'Chaos')],)
colnames(a2) = c('sound', 'measure', 'Manipulation', 'observed')

ggplot(contrasts, aes(x = sound, y = fit, ymin = lwr, ymax = upr)) +
  geom_errorbar(width = 0) +
  geom_point() +
  geom_point(data = a2, aes(x = sound, y = observed), inherit.aes = FALSE, shape = 4) +
  geom_hline(yintercept = 0, linetype = 3) +
  coord_flip() +
  facet_wrap(~measure + Manipulation) +
  theme_bw()
